% blks
blk(a).
blk(b).
blk(c).

% locations
loc(table).
loc(B) :- 
	blk(B).

% fluents
fluent(on(B,L)) :-
	blk(B), loc(L).
fluent(occupied(B)) :- 
	loc(B).

% actions
% move a blk to another place (on another blk
% or on the table)
action(move(B,L)) :-
	blk(B), loc(L).

% executability conditions
executable(move(B,L),[neg(occupied(B)),neg(occupied(L))]) :-
	blk(B), loc(L), B \== L.

% dynamic laws
causes(move(B,L),on(B,L),[]) :-
	blk(B), loc(L).
causes(move(B,L),neg(on(B,L1)),[on(B,L1)]) :-
	blk(B), loc(L), loc(L1), L \== L1.
causes(move(B,L),neg(occupied(L1)),[on(B,L1)]) :-
	blk(B), loc(L), loc(L1), L \== L1.

% static laws
caused(occupied(B),[on(B1,B)]) :-
	blk(B), blk(B1).

% initial state
initially(neg(occupied(table))).
initially(neg(occupied(c))).
initially(neg(occupied(b))).
initially(on(a,table)).
initially(on(b,table)).
initially(on(c,a)).

% goal
goal(on(c,b)).
goal(on(b,a)).
goal(on(a,table)).