% MEDICAL EXAMPLE

% illnesses
ill(1).
ill(2).
ill(3).
ill(4).
ill(5).

% fluents
fluent(infected(I)) :- 
    ill(I).
fluent(infected(none)).
fluent(tcd).
fluent(bsd).
fluent(hc).
fluent(dead).
fluent(color(red)).
fluent(color(blue)).
fluent(color(white)).

% actions
action(inspect).
action(stain).
action(blood_sample).
action(analyze_blood).
action(medicate(X)):- 
    ill(X).

% executability conditions
executable(inspect,[tcd,neg(dead)]). 
executable(stain,[neg(dead)]).
executable(blood_sample,[neg(dead)]). 
executable(analyze_blood,[bsd,neg(dead)]). 
executable(medicate(X),[neg(dead)]):-
    ill(X).


% dynamic laws
causes(stain,tcd,[]).

% the patient will get cured if the right medication
% is taken
causes(medicate(I),infected(none),[infected(I)]):-
      ill(I).

% the patient will be dead if a wrong medication
% is taken
causes(medicate(I),dead,[infected(J)]):-
      ill(I), ill(J), I \== J.

causes(blood_sample,bsd,[]). 

% k-propositions
determines(inspect,[color(red),color(blue),color(white)]).
determines(analyze_blood,[hc,neg(hc)]).    

% static laws
oneof([infected(none),infected(1),infected(2),
       infected(3),infected(4),infected(5)]).

caused(color(blue),[tcd,infected(3)]).
caused(color(blue),[tcd,infected(4)]).

caused(color(red),[tcd,infected(1)]).
caused(color(red),[tcd,infected(2)]).

caused(color(white),[tcd,infected(5)]).

caused(hc,[bsd,infected(1)]).
caused(hc,[bsd,infected(3)]).
caused(hc,[bsd,infected(5)]).

caused(neg(hc),[bsd,infected(2)]).
caused(neg(hc),[bsd,infected(4)]).

% The patient has exactly one illness
initially(infected(1)).

initially(neg(tcd)).
initially(neg(bsd)).
initially(neg(dead)).

goal(infected(none)).
goal(neg(dead)).
