% This is the Airport Problem from (Levesque 1996)

% locations
gate(gateA).
gate(gateB).
loc(airport).
loc(X) :- 
	gate(X).

plane(f123).

% fluents
fluent(at(L)) :-
	loc(L).
fluent(at(home)).
fluent(on_plane(P)) :-
	plane(P).
fluent(parked(P,G)) :-
	plane(P), gate(G).

% actions
action(drive).
action(walk(L)) :-
	loc(L).
action(board_plane(P)) :-
	plane(P).
action(check_departures).

% executability conditions
executable(drive,[at(home)]).
executable(walk(L),[at(L1)]) :-
	loc(L), loc(L1), L \== L1.
executable(board_plane(P),[at(G),parked(P,G)]) :-
	plane(P), gate(G).
executable(check_departures,[at(airport)]).

% dynamic laws
causes(drive,at(airport),[]).
causes(walk(L),at(L),[]) :-
	loc(L).
causes(board_plane(P),on_plane(P),[]) :-
	plane(P).
determines(check_departures,
	[parked(P,gateA),parked(P,gateB)]) :-
	plane(P).

% static laws
oneof(Ls) :-
	findall(at(L),loc(L),Ls1), 
	append(Ls1,[at(home)],Ls).

oneof([parked(P,gateA),parked(P,gateB)]) :-
	plane(P).

% initial state
initially(at(home)).

% goal
goal(on_plane(f123)).