% blks
blk(b0).
blk(b1).
blk(b2).
blk(b3).
blk(b4).
blk(b5).
blk(b6).
blk(b7).

% locations
loc(table).
loc(B) :- 
	blk(B).

% fluents
fluent(on(B,L)) :-
	blk(B), loc(L).
fluent(occupied(B)) :- 
	loc(B).

% actions
% move a blk to another place (on another blk
% or on the table)
action(move(B,L)) :-
	blk(B), loc(L).

% executability conditions
executable(move(B,L),[neg(occupied(B)),neg(occupied(L))]) :-
	blk(B), loc(L), B \== L.

% dynamic laws
causes(move(B,L),on(B,L),[]) :-
	blk(B), loc(L).
causes(move(B,L),neg(on(B,L1)),[on(B,L1)]) :-
	blk(B), loc(L), loc(L1), L \== L1.
causes(move(B,L),neg(occupied(L1)),[on(B,L1)]) :-
	blk(B), loc(L), loc(L1), L \== L1.

% static laws
caused(occupied(B),[on(B1,B)]) :-
	blk(B), blk(B1).

% initial state
initially(neg(occupied(table))).
initially(neg(occupied(b2))).
initially(neg(occupied(b4))).
initially(neg(occupied(b7))).

initially(on(b0,table)).
initially(on(b3,b0)).
initially(on(b2,b3)).

initially(on(b1,table)).
initially(on(b4,b1)).

initially(on(b5,table)).
initially(on(b6,b5)).
initially(on(b7,b6)).


% goal
goal(on(b4,table)).
goal(on(b3,b4)).
goal(on(b7,b3)).

goal(on(b6,table)).
goal(on(b2,b6)).

goal(on(b1,table)).
goal(on(b0,b1)).
goal(on(b5,b0)).
