% This is a variation of the Monkey and Banana
% Problem described in the CCalc manual at
% http://www.cs.utexas.edu/users/mccain/cc/

% object types
object(monkey).
object(box).
object(bananas).
loc(1).
loc(2).
loc(3).

% fluents
fluent(at(X,L)) :- 
	object(X), loc(L).
fluent(on_box).
fluent(has_bananas).

% actions
action(walk(L))  :- 
	loc(L).
action(push_box(L)) :- 
	loc(L).
action(climb_box).
action(grasp_bananas).

% executability conditions
executable(walk(L),[neg(on_box)]) :- 
	loc(L).
executable(push_box(L),[at(monkey,L1),at(box,L1),neg(on_box)]) :- 
	loc(L), loc(L1).
executable(climb_box,[at(monkey,L),at(box,L),neg(on_box)]) :-
	loc(L).
executable(grasp_bananas,[at(monkey,L),at(bananas,L),on_box]) :-
	loc(L).

% dynamic laws
causes(walk(L),at(monkey,L),[]) :- 
	loc(L).
causes(push_box(L),at(monkey,L),[]) :- 
	loc(L).
causes(push_box(L),at(box,L),[]) :- 
	loc(L).
causes(climb_box,on_box,[]).
causes(grasp_bananas,has_bananas,[]).

% static laws
oneof([at(O,1),at(O,2),at(O,3)]) :-
	object(O).

% initial state
initially(at(monkey,1)).
initially(at(box,2)).
initially(at(bananas,3)).
initially(neg(has_bananas)).
initially(neg(on_box)).

% goal
goal(has_bananas).
