% MEDICAL EXAMPLE (from Weld et al. 1998)
% In this problem, we know the illness of the patient
% is either i1 or i2.

% illnesses
ill(1).
ill(2).
ill(3).
ill(4).
ill(5).

% fluents
fluent(infected(I)) :- 
    ill(I).
fluent(infected(none)).
fluent(tcd).
fluent(bsd).
fluent(hc).
fluent(dead).
fluent(color(red)).
fluent(color(blue)).
fluent(color(white)).

% actions
action(inspect).
action(stain).
action(blood_sample).
action(analyze_blood).
action(medicate(X)):- 
    ill(X).

% executability conditions
executable(inspect,[tcd,neg(dead)]). 
executable(stain,[neg(dead)]).
executable(blood_sample,[neg(dead)]). 
executable(analyze_blood,[bsd,neg(dead)]). 
executable(medicate(X),[neg(dead)]):-
    ill(X).


% dynamic laws
causes(stain,tcd,[]).

% the patient will get cured if the right medication
% is taken
causes(medicate(I),infected(none),[infected(I)]):-
      ill(I).

% the patient will be dead if a wrong medication
% is taken
causes(medicate(I),dead,[infected(J)]):-
      ill(I), ill(J), I \== J.

causes(blood_sample,bsd,[]). 

% k-propositions
determines(inspect,[color(red),color(blue),color(white)]).
determines(analyze_blood,[hc,neg(hc)]).    

% static laws
oneof([infected(none),infected(1),infected(2),
       infected(3),infected(4),infected(5)]).


caused(neg(infected(none)),[tcd,color(blue)]).
caused(neg(infected(1)),[tcd,color(blue)]).
caused(neg(infected(2)),[tcd,color(blue)]).
caused(neg(infected(5)),[tcd,color(blue)]).

caused(neg(infected(none)),[tcd,color(red)]).
caused(neg(infected(3)),[tcd,color(red)]).
caused(neg(infected(4)),[tcd,color(red)]).
caused(neg(infected(5)),[tcd,color(red)]).

caused(infected(5),[tcd,color(white)]).

caused(neg(infected(none)),[bsd,hc]).
caused(neg(infected(2)),[bsd,hc]).
caused(neg(infected(4)),[bsd,hc]).

caused(neg(infected(none)),[bsd,neg(hc)]).
caused(neg(infected(1)),[bsd,neg(hc)]).
caused(neg(infected(3)),[bsd,neg(hc)]).
caused(neg(infected(5)),[bsd,neg(hc)]).

% initial
initially(neg(tcd)).
initially(neg(bsd)).
initially(neg(dead)).

initially(neg(infected(3))).
initially(neg(infected(4))).
initially(neg(infected(5))).
initially(neg(infected(none))).

% goal
goal(infected(none)).
goal(neg(dead)).
