% SICK EXAMPLE: This is a variation of
% the sick domain from (Weld et al 1998)

% kinds of illnesses
ill(1).
ill(2).
ill(3).
ill(4).
ill(5).
ill(6).
ill(7).
ill(8).

% fluents
fluent(infected(I)) :- 
    ill(I).
fluent(dead).
fluent(tcd).
fluent(color(I)) :- 
    ill(I).
fluent(none).

% actions
action(inspect).
action(stain).
action(medicate(X)):- 
    ill(X).

% executability conditions
executable(inspect,[neg(dead)]). 
executable(stain,[neg(dead)]).
executable(medicate(X),[neg(dead)]):-
    ill(X).

% dynamic laws
causes(stain,tcd,[]).

% the patient will get cured if the right medication
% is taken
causes(medicate(I),none,[infected(I)]):-
      ill(I).

% the patient will be dead if a wrong medication
% is taken
causes(medicate(I),dead,[infected(J)]):-
      ill(I), ill(J), I \== J.

% k-propositions
determines(inspect,Colors) :-
    findall(color(X),ill(X),Colors).

% static laws
oneof([none|Ills]) :-
    findall(infected(X),ill(X),Ills).

caused(infected(X),[color(X),tcd]) :-
    ill(X).

% initial
initially(neg(tcd)).
initially(neg(none)).
initially(neg(dead)).

% goal
goal(none).
goal(neg(dead)).
