(in-package :domains)

;;; This file contains some bomb in the toilet domains 
;;;  - bt       Bomb in the toilet with single toilet, no clogging
;;;  - bmt      Bomb in the toilet with multiple toilets, no clogging
;;;  - btc      Bomb in the toilet with clogging
;;;  - bmtc     Bomb in the toilet with multiple toilets & clogging
;;;  - bts1     Bomb in the toilet with one sensing action
;;;  - bts2     Bomb in the toilet with two sensing actions
;;;  - bts3     Bomb in the toilet with three sensing actions
;;;  - bts4     Bomb in the toilet with four sensing actions

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BT: Bomb in the toilet with single toilet, no clogging
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bt)
  (:requirements :strips :equality :typing :conditional-effects :uncertainty)
  (:action dunk
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (and 
		    (when (in ?p) (defused))
		    (dunked ?p))))

;; Problem: BT(2)
(define (problem bt_2)             
  (:domain bt)
  (:objects p1 p2)
  (:init (package p1) (package p2) (not (dunked p1)) (not (dunked p2)) 
         (oneof (in p1) (in p2)))
  (:goal (defused)))

;; Problem: BT(4)
(define (problem bt_4)
  (:domain bt)
  (:objects p1 p2 p3 p4)
  (:init (package p1) (package p2) (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
         (oneof (in p1) (in p2) (in p3) (in p4)))
  (:goal (defused)))

;; Problem: BT(6)
(define (problem bt_6)             
  (:domain bt)
  (:objects p1 p2 p3 p4 p5 p6)
  (:init (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
         (oneof (in p1) (in p2) (in p3) (in p4) (in p5) (in p6)))
  (:goal (defused)))

;; Problem: BT(8)
(define (problem bt_8)             
  (:domain bt)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8)
  (:init (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)))
  (:goal (defused)))

;; Problem: BT(10)
(define (problem bt_10)             
  (:domain bt)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10)
  (:init (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8) (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
	 (not (dunked p9)) (not (dunked p10)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BT ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BMT: Bomb in the toilet with multiple toilets, no clogging
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bmt)
  (:requirements :strips :equality :typing :conditional-effects :uncertainty)
  (:action dunk
             :parameters (?p ?t)
             :precondition (and (toilet ?t) (package ?p) 
                                (not (dunked ?p)))
             :effect (and (dunked ?p) 
			  (when (in ?p) (defused)))))


;; Problem: BMT(2,2)
(define (problem bmt_2_2)
  (:domain bmt)
  (:objects p1 p2 t1 t2)
  (:init (toilet t1) (toilet t2)
	 (package p1) (package p2) 
	 (not (dunked p1)) (not (dunked p2)) 
         (oneof (in p1) (in p2)))
  (:goal (defused)))

;; Problem: BMT(4,2)
(define (problem bmt_4_2)              ; 
  (:domain bmt)
  (:objects p1 p2 p3 p4 t1 t2)
  (:init (toilet t1) (toilet t2)
	 (package p1) (package p2) (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
         (oneof (in p1) (in p2) (in p3) (in p4)))
  (:goal (defused)))

;; Problem: BMT(6,2)
(define (problem bmt_6_2)             
  (:domain bmt)
  (:objects p1 p2 p3 p4 p5 p6 t1 t2)
  (:init (toilet t1) (toilet t2)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
         (oneof (in p1) (in p2) (in p3) (in p4) (in p5) (in p6)))
  (:goal (defused)))

;; Problem: BMT(8,4)
(define (problem bmt_8_4)             
  (:domain bmt)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 t1 t2 t3 t4)
  (:init (toilet t1) (toilet t2)
	 (toilet t3) (toilet t4)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)))
  (:goal (defused)))

;; Problem: BMT(10,4)
(define (problem bmt_10_4)             
  (:domain bmt)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10
	    t1 t2 t3 t4)
  (:init (toilet t1) (toilet t2)
	 (toilet t3) (toilet t4)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8) (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
	 (not (dunked p9)) (not (dunked p10)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BMT ;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BTC: Bomb in the toilet with single toilet, clogging
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain btc)
  (:requirements :conditional-effects :uncertainty)
  (:action dunk
             :parameters (?p)
             :precondition (and (package ?p) (not (clogged)) 
				(not (dunked ?p))) 
             :effect (and (clogged) (dunked ?p)
		           (when (in ?p)
		             (defused))))
  (:action flush
             :parameters ()
	     :precondition (clogged)
             :effect (not (clogged))))

;; Problem: BTC(2)
(define (problem btc_2)              ; Plan length = 2
  (:domain btc)
  (:objects p1 p2)
  (:init (not (clogged))(package p1) (package p2) 
	 (not (dunked p1)) (not (dunked p2)) 
         (oneof (in p1) (in p2)))
  (:goal (defused)))

;; Problem: BTC(4)
(define (problem btc_4)              ; 
  (:domain btc)
  (:objects p1 p2 p3 p4)
  (:init (not (clogged))(package p1) (package p2) (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
         (oneof (in p1) (in p2) (in p3) (in p4)))
  (:goal (defused)))

;; Problem: BTC(6)
(define (problem btc_6)             
  (:domain btc)
  (:objects p1 p2 p3 p4 p5 p6)
  (:init (not (clogged))(package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
         (oneof (in p1) (in p2) (in p3) (in p4) (in p5) (in p6)))
  (:goal (defused)))

;; Problem: BTC(8)
(define (problem btc_8)             
  (:domain btc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8)
  (:init (not (clogged))(package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)))
  (:goal (defused)))

;; Problem: BTC(10)
(define (problem btc_10)             
  (:domain btc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10)
  (:init (not (clogged))(package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8) (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
	 (not (dunked p9)) (not (dunked p10)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BTC ;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BMTC: Bomb in the toilet with multiple toilets, clogging
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bmtc)
  (:requirements :strips :equality :typing :conditional-effects :uncertainty)
  (:action dunk
             :parameters (?p ?t)
             :precondition (and (package ?p) (toilet ?t)
                                (not (dunked ?p)) (not (clogged ?t)))
             :effect (and (clogged ?t) (dunked ?p)
		           (when (in ?p)
		             (defused))))
  (:action flush
             :parameters (?t)
             :precondition (and (toilet ?t))
             :effect (not (clogged ?t))))

;; Problem: BMTC(2,2)
(define (problem bmtc_2_2)
  (:domain bmtc)
  (:objects p1 p2 t1 t2)
  (:init (not (clogged t1)) (not (clogged t2))
	 (toilet t1) (toilet t2)
	 (package p1) (package p2)
	 (not (dunked p1)) (not (dunked p2)) 
         (oneof (in p1) (in p2)))
  (:goal (defused)))

;; Problem: BMTC(4,2)
(define (problem bmtc_4_2)              ; 
  (:domain bmtc)
  (:objects p1 p2 p3 p4 t1 t2)
  (:init (not (clogged t1)) (not (clogged t2))
	 (toilet t1) (toilet t2)
	 (package p1) (package p2) (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
         (oneof (in p1) (in p2) (in p3) (in p4)))
  (:goal (defused)))

;; Problem: BMTC(6)
(define (problem bmtc_6_2)
  (:domain bmtc)
  (:objects p1 p2 p3 p4 p5 p6 t1 t2)
  (:init (not (clogged t1)) (not (clogged t2))
	 (toilet t1) (toilet t2)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
         (oneof (in p1) (in p2) (in p3) (in p4) (in p5) (in p6)))
  (:goal (defused)))

;; Problem: BMTC(8,4)
(define (problem bmtc_8_4)             
  (:domain bmtc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 t1 t2 t3 t4)
  (:init (not (clogged t1)) (not (clogged t2))
	 (not (clogged t3)) (not (clogged t4))
	 (toilet t1) (toilet t2)
	 (toilet t3) (toilet t4)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)))
  (:goal (defused)))

;; Problem: BMTC(10,4)
(define (problem bmtc_10_4)             
  (:domain bmtc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10
	    t1 t2 t3 t4)
  (:init (not (clogged t1)) (not (clogged t2))
	 (not (clogged t3)) (not (clogged t4))
	 (toilet t1) (toilet t2)
	 (toilet t3) (toilet t4)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8) (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
	 (not (dunked p9)) (not (dunked p10)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BMTC ;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BTS1: Bomb in the toilet with clogging & one sensing action
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bts1) (:requirements :typing)
  (:action dunk
	   :parameters (?p)
	   :precondition (and (package ?p) (not (clogged)) 
			      (not (dunked ?p)))
	   :effect (and (clogged)
			(when (in ?p) (defused))))
  
  (:action detect_metal
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (observes (metal ?p)))
)

;; Problem: BTS1(2)
(define (problem bts1_2)
  (:domain bts1)
  (:objects p1 p2)
  (:init (package p1) (package p2)
	 (not (dunked p1)) (not (dunked p2))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2)))
  (:goal (defused)))

;; Problem: BTS1(4)
(define (problem bts1_4)
  (:domain bts1)
  (:objects p1 p2 p3 p4)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4))
	 (oneof (in p1) (in p2))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4)))
  (:goal (defused)))

;; Problem: BTS1(6)
(define (problem bts1_6)
  (:domain bts1)
  (:objects p1 p2 p3 p4 p5 p6)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6)))
  (:goal (defused)))

;; Problem: BTS1(8)
(define (problem bts1_8)
  (:domain bts1)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (dunked p7)) (not (dunked p8))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6) (in p7) (in p8))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p5) (metal p5))
	 (iff (in p7) (metal p8)))
  (:goal (defused)))

;; Problem: BTS1(10)
(define (problem bts1_10)
  (:domain bts1)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (package p7) (package p8)
	 (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (dunked p7)) (not (dunked p8))
	 (not (dunked p9)) (not (dunked p10))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p5) (metal p5))
	 (iff (in p7) (metal p8))
	 (iff (in p9) (metal p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BTS1 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BTS2: Bomb in the toilet with clogging & two sensing actions:
;;              detect_metal & sniff
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bts2) (:requirements :typing)
  (:action dunk
	   :parameters (?p)
	   :precondition (and (package ?p) (not (clogged)) 
			      (not (dunked ?p)))
	   :effect (and (clogged)
			(when (in ?p) (defused))))
  
  (:action detect_metal
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (observes (metal ?p)))

  (:action sniff
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (observes (odor_of_explosives ?p)))
)

;; Problem: BTS2(2)
(define (problem bts2_2)
  (:domain bts2)
  (:objects p1 p2)
  (:init (package p1) (package p2)
	 (not (dunked p1)) (not (dunked p2))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2)))
  (:goal (defused)))

;; Problem: BTS2(4)
(define (problem bts2_4)
  (:domain bts2)
  (:objects p1 p2 p3 p4)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4))
	 (oneof (in p1) (in p2))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4)))
  (:goal (defused)))

;; Problem: BTS2(6)
(define (problem bts2_6)
  (:domain bts2)
  (:objects p1 p2 p3 p4 p5 p6)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p6) (odor_of_explosives p6)))
  (:goal (defused)))

;; Problem: BTS2(8)
(define (problem bts2_8)
  (:domain bts2)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (dunked p7)) (not (dunked p8))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6) (in p7) (in p8))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p5) (metal p5))
	 (iff (in p7) (metal p8))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p6) (odor_of_explosives p6))
	 (iff (in p7) (odor_of_explosives p7))
	 (iff (in p8) (odor_of_explosives p8)))
  (:goal (defused)))

;; Problem: BTS2(10)
(define (problem bts2_10)
  (:domain bts2)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (package p7) (package p8)
	 (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (dunked p7)) (not (dunked p8))
	 (not (dunked p9)) (not (dunked p10))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p5) (metal p5))
	 (iff (in p7) (metal p8))
	 (iff (in p9) (metal p10))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p6) (odor_of_explosives p6))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p7) (odor_of_explosives p8))
	 (iff (in p9) (odor_of_explosives p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BTS2 ;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BTS3: Bomb in the toilet with clogging & three sensing 
;;              actions: detect_metal, sniff, and xray
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bts3) (:requirements :typing)
  (:action dunk
	   :parameters (?p)
	   :precondition (and (package ?p) (not (clogged)) 
			      (not (dunked ?p)))
	   :effect (and (clogged)
			(when (in ?p) (defused))))
  
  (:action detect_metal
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (observes (metal ?p)))

  (:action sniff
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (observes (odor_of_explosives ?p)))

  (:action xray
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (observes (dense_material ?p))))

;; Problem: BTS3(2)
(define (problem bts3_2)
  (:domain bts3)
  (:objects p1 p2)
  (:init (package p1) (package p2)
	 (not (dunked p1)) (not (dunked p2))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p1) (dense_material p1))
	 (iff (in p2) (dense_material p2)))
  (:goal (defused)))

;; Problem: BTS3(4)
(define (problem bts3_4)
  (:domain bts3)
  (:objects p1 p2 p3 p4)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4))
	 (oneof (in p1) (in p2))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p1) (dense_material p1))
	 (iff (in p2) (dense_material p2))
	 (iff (in p3) (dense_material p3))
	 (iff (in p4) (dense_material p4))
	 )
  (:goal (defused)))

;; Problem: BTS3(6)
(define (problem bts3_6)
  (:domain bts3)
  (:objects p1 p2 p3 p4 p5 p6)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p6) (odor_of_explosives p6))
	 (iff (in p1) (dense_material p1))
	 (iff (in p2) (dense_material p2))
	 (iff (in p3) (dense_material p3))
	 (iff (in p4) (dense_material p4))
	 (iff (in p5) (dense_material p5))
	 (iff (in p6) (dense_material p6))
)
  (:goal (defused)))

;; Problem: BTS3(8)
(define (problem bts3_8)
  (:domain bts3)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (dunked p7)) (not (dunked p8))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6) (in p7) (in p8))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p5) (metal p5))
	 (iff (in p7) (metal p8))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p6) (odor_of_explosives p6))
	 (iff (in p7) (odor_of_explosives p7))
	 (iff (in p8) (odor_of_explosives p8))
	 (iff (in p1) (dense_material p1))
	 (iff (in p2) (dense_material p2))
	 (iff (in p3) (dense_material p3))
	 (iff (in p4) (dense_material p4))
	 (iff (in p5) (dense_material p5))
	 (iff (in p6) (dense_material p6))
	 (iff (in p7) (dense_material p7))
	 (iff (in p8) (dense_material p8))
)
  (:goal (defused)))

;; Problem: BTS3(10)
(define (problem bts3_10)
  (:domain bts3)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (package p7) (package p8)
	 (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (dunked p7)) (not (dunked p8))
	 (not (dunked p9)) (not (dunked p10))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p5) (metal p5))
	 (iff (in p7) (metal p8))
	 (iff (in p9) (metal p10))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p6) (odor_of_explosives p6))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p7) (odor_of_explosives p8))
	 (iff (in p9) (odor_of_explosives p10))
	 (iff (in p1) (dense_material p1))
	 (iff (in p2) (dense_material p2))
	 (iff (in p3) (dense_material p3))
	 (iff (in p4) (dense_material p4))
	 (iff (in p5) (dense_material p5))
	 (iff (in p6) (dense_material p6))
	 (iff (in p5) (dense_material p5))
	 (iff (in p7) (dense_material p8))
	 (iff (in p9) (dense_material p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BTS3 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BTS4: Bomb in the toilet with clogging & three sensing 
;;      actions: detect_metal, sniff, xray, & listen_for_ticking
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bts4) (:requirements :typing)
  (:action dunk
	   :parameters (?p)
	   :precondition (and (package ?p) (not (clogged)) 
			      (not (dunked ?p)))
	   :effect (and (clogged)
			(when (in ?p) (defused))))
  
  (:action detect_metal
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (observes (metal ?p)))

  (:action sniff
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (observes (odor_of_explosives ?p)))

  (:action xray
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (observes (dense_material ?p)))

  (:action listen_for_ticking
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p)))
	   :effect (observes (ticking ?p))))

;; Problem: BTS4(2)
(define (problem bts4_2)
  (:domain bts4)
  (:objects p1 p2)
  (:init (package p1) (package p2)
	 (not (dunked p1)) (not (dunked p2))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p1) (dense_material p1))
	 (iff (in p2) (dense_material p2))
	 (iff (in p1) (ticking p1))
	 (iff (in p2) (ticking p2)))
  (:goal (defused)))

;; Problem: BTS4(4)
(define (problem bts4_4)
  (:domain bts4)
  (:objects p1 p2 p3 p4)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4))
	 (oneof (in p1) (in p2))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p1) (dense_material p1))
	 (iff (in p2) (dense_material p2))
	 (iff (in p3) (dense_material p3))
	 (iff (in p4) (dense_material p4))
	 (iff (in p1) (ticking p1))
	 (iff (in p2) (ticking p2))
	 (iff (in p3) (ticking p3))
	 (iff (in p4) (ticking p4)))
  (:goal (defused)))

;; Problem: BTS4(6)
(define (problem bts4_6)
  (:domain bts4)
  (:objects p1 p2 p3 p4 p5 p6)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p6) (odor_of_explosives p6))
	 (iff (in p1) (dense_material p1))
	 (iff (in p2) (dense_material p2))
	 (iff (in p3) (dense_material p3))
	 (iff (in p4) (dense_material p4))
	 (iff (in p5) (dense_material p5))
	 (iff (in p6) (dense_material p6))
	 (iff (in p1) (ticking p1))
	 (iff (in p2) (ticking p2))
	 (iff (in p3) (ticking p3))
	 (iff (in p4) (ticking p4))
	 (iff (in p5) (ticking p5))
	 (iff (in p6) (ticking p6)))
  (:goal (defused)))

;; Problem: BTS4(8)
(define (problem bts4_8)
  (:domain bts4)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (dunked p7)) (not (dunked p8))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6) (in p7) (in p8))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p5) (metal p5))
	 (iff (in p7) (metal p8))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p6) (odor_of_explosives p6))
	 (iff (in p7) (odor_of_explosives p7))
	 (iff (in p8) (odor_of_explosives p8))
	 (iff (in p1) (dense_material p1))
	 (iff (in p2) (dense_material p2))
	 (iff (in p3) (dense_material p3))
	 (iff (in p4) (dense_material p4))
	 (iff (in p5) (dense_material p5))
	 (iff (in p6) (dense_material p6))
	 (iff (in p7) (dense_material p7))
	 (iff (in p8) (dense_material p8))
	 (iff (in p1) (ticking p1))
	 (iff (in p2) (ticking p2))
	 (iff (in p3) (ticking p3))
	 (iff (in p4) (ticking p4))
	 (iff (in p5) (ticking p5))
	 (iff (in p6) (ticking p6))
	 (iff (in p7) (ticking p7))
	 (iff (in p8) (ticking p8)))
  (:goal (defused)))

;; Problem: BTS4(10)
(define (problem bts4_10)
  (:domain bts4)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10)
  (:init (package p1) (package p2)
	 (package p3) (package p4)
	 (package p5) (package p6)
	 (package p7) (package p8)
	 (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2))
	 (not (dunked p3)) (not (dunked p4))
	 (not (dunked p5)) (not (dunked p6))
	 (not (dunked p7)) (not (dunked p8))
	 (not (dunked p9)) (not (dunked p10))
	 (not (clogged))
	 (not (defused))
	 (oneof (in p1) (in p2) (in p3) (in p4)
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10))
	 (iff (in p1) (metal p1))
	 (iff (in p2) (metal p2))
	 (iff (in p3) (metal p3))
	 (iff (in p4) (metal p4))
	 (iff (in p5) (metal p5))
	 (iff (in p6) (metal p6))
	 (iff (in p5) (metal p5))
	 (iff (in p7) (metal p8))
	 (iff (in p9) (metal p10))
	 (iff (in p1) (odor_of_explosives p1))
	 (iff (in p2) (odor_of_explosives p2))
	 (iff (in p3) (odor_of_explosives p3))
	 (iff (in p4) (odor_of_explosives p4))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p6) (odor_of_explosives p6))
	 (iff (in p5) (odor_of_explosives p5))
	 (iff (in p7) (odor_of_explosives p8))
	 (iff (in p9) (odor_of_explosives p10))
	 (iff (in p1) (dense_material p1))
	 (iff (in p2) (dense_material p2))
	 (iff (in p3) (dense_material p3))
	 (iff (in p4) (dense_material p4))
	 (iff (in p5) (dense_material p5))
	 (iff (in p6) (dense_material p6))
	 (iff (in p5) (dense_material p5))
	 (iff (in p7) (dense_material p8))
	 (iff (in p9) (dense_material p10))
	 (iff (in p1) (ticking p1))
	 (iff (in p2) (ticking p2))
	 (iff (in p3) (ticking p3))
	 (iff (in p4) (ticking p4))
	 (iff (in p5) (ticking p5))
	 (iff (in p6) (ticking p6))
	 (iff (in p5) (ticking p5))
	 (iff (in p7) (ticking p8))
	 (iff (in p9) (ticking p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BTS4 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BTUC: Bomb in the toilet with single toilet, clogging
;;               & uncertainty in clogging in the beginning
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain btuc)
  (:requirements :strips :equality :typing :conditional-effects :uncertainty)
  (:action dunk
             :parameters (?p)
             :precondition (and (package ?p) (not (clogged)) 
				(not (dunked ?p))) 
             :effect (and (clogged) (dunked ?p)
		           (when (in ?p)
		             (defused))))
  (:action flush
             :parameters ()
	     :precondition (clogged)
             :effect (not (clogged))))

;; Problem: BTUC(2)
(define (problem btuc_2)              ; Plan length = 2
  (:domain btuc)
  (:objects p1 p2)
  (:init (uncertain (clogged))
	 (package p1) (package p2) 
	 (not (dunked p1)) (not (dunked p2)) 
         (oneof (in p1) (in p2)))
  (:goal (defused)))

;; Problem: BTUC(4)
(define (problem btuc_4)              ; 
  (:domain btuc)
  (:objects p1 p2 p3 p4)
  (:init (uncertain (clogged))
	 (package p1) (package p2) (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
         (oneof (in p1) (in p2) (in p3) (in p4)))
  (:goal (defused)))

;; Problem: BTUC(6)
(define (problem btuc_6)             
  (:domain btuc)
  (:objects p1 p2 p3 p4 p5 p6)
  (:init (uncertain (clogged))	 
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
         (oneof (in p1) (in p2) (in p3) (in p4) (in p5) (in p6)))
  (:goal (defused)))

;; Problem: BTUC(8)
(define (problem btuc_8)             
  (:domain btuc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8)
  (:init (uncertain clogged)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)))
  (:goal (defused)))

;; Problem: BTUC(10)
(define (problem btuc_10)             
  (:domain btuc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10)
  (:init (uncertain clogged)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8) (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
	 (not (dunked p9)) (not (dunked p10)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BTUC ;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BMTUC: Bomb in the toilet with multiple toilets, clogging
;;               & uncertainty in clogging in the beginning
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bmtuc)
  (:requirements :strips :equality :typing :conditional-effects :uncertainty)
  (:action dunk
             :parameters (?p ?t)
             :precondition (and (package ?p) (toilet ?t)
                                (not (dunked ?p)) (not (clogged ?t)))
             :effect (and (clogged ?t) (dunked ?p)
		           (when (in ?p)
		             (defused))))
  (:action flush
             :parameters (?t)
             :precondition (and (toilet ?t))
             :effect (not (clogged ?t))))

;; Problem: BMTUC(2,2)
(define (problem bmtuc_2_2)
  (:domain bmtuc)
  (:objects p1 p2 t1 t2)
  (:init (uncertain (clogged t1)) (uncertain (clogged t2))
	 (toilet t1) (toilet t2)
	 (package p1) (package p2)
	 (not (dunked p1)) (not (dunked p2)) 
         (oneof (in p1) (in p2)))
  (:goal (defused)))

;; Problem: BMTUC(4,2)
(define (problem bmtuc_4_2)              ; 
  (:domain bmtuc)
  (:objects p1 p2 p3 p4 t1 t2)
  (:init (uncertain (clogged t1)) (uncertain (clogged t2))
	 (toilet t1) (toilet t2)
	 (package p1) (package p2) (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
         (oneof (in p1) (in p2) (in p3) (in p4)))
  (:goal (defused)))

;; Problem: BMTUC(6)
(define (problem bmtuc_6_2)
  (:domain bmtuc)
  (:objects p1 p2 p3 p4 p5 p6 t1 t2)
  (:init (uncertain (clogged t1)) (uncertain (clogged t2))
	 (toilet t1) (toilet t2)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
         (oneof (in p1) (in p2) (in p3) (in p4) (in p5) (in p6)))
  (:goal (defused)))

;; Problem: BMTUC(8,4)
(define (problem bmtuc_8_4)             
  (:domain bmtuc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 t1 t2 t3 t4)
  (:init (uncertain (clogged t1)) (uncertain (clogged t2))
	 (uncertain (clogged t3)) (uncertain (clogged t4))
	 (toilet t1) (toilet t2)
	 (toilet t3) (toilet t4)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)))
  (:goal (defused)))

;; Problem: BMTUC(10,4)
(define (problem bmtuc_10_4)             
  (:domain bmtuc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10
	    t1 t2 t3 t4)
  (:init (uncertain (clogged t1)) (uncertain (clogged t2))
	 (uncertain (clogged t3)) (uncertain (clogged t4))
	 (toilet t1) (toilet t2)
	 (toilet t3) (toilet t4)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8) (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
	 (not (dunked p9)) (not (dunked p10)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BMTUC ;;;;;;;;;;;;;;;;;;;;;;;;;;;;