;; This is MED domain, from Weld et al. 1998
(in-package :domains)

(define (domain med)
    (:requirements :conditional-effects :sensing)
  
  ;; There are 5 different kinds of illnesses.
  (:constants I1 I2 I3 I4 I5)

  ;; We can perform a throat culture.  The culture
  ;; will turn blue, red, or white
  (:action stain
	   :parameters ()
	   :precondition ()
	   :effect (and (when (or (I3) (I4)) (B))
			(when (or (I1) (I2)) (R))
			(when (I5) (W))))
  (:action inspect
	   :parameters ()
	   :precondition ()
	   :effect (and (observes (B))
			(observes (R))
			(observes (W))))
  
  ;; We can take a blood sample.  The result will be whether the 
  ;; patient has a high white cell count.
  (:action count-white-cells
	   :effect (and (when (or (I1) (I3) (I5)) (high-white-cell-count))))

  (:action analyze-blood
	   :effect (observes (high-white-cell-count)))
   
	   
  ;; We can cure any illness, but using the wrong cure is fatal.
  (:action medicate
	   :parameters (?ill)
	   :precondition (object ?ill)
	   :effect (and (when (?ill) (not (?ill)))
			(when (not (?ill)) (D)))))

(define (problem med1)
    (:domain med)
  
  (:init (not (R)) (not (B)) (not (W))	; Culture hasn't been done yet
	 (not (high-white-cell-count))	; Blood sample hasn't been taken.

	 ;; The patient has exactly one illness
	 (I1)
	 (not (I2)) (not (I3)) (not (I4)) (not (I5)) 

	 ;; The patient isn't dead yet.
	 (not (D)))
  (:goal (and (not (I1)) (not (I2)) (not (I3)) (not (I4)) (not (I5))
	      (not (D)))))

(define (problem med2)
    (:domain med)
  
  (:init (not (R)) (not (B)) (not (W))	; Culture hasn't been done yet
	 (not (high-white-cell-count))	; Blood sample hasn't been taken.

	 ;; The patient has exactly one illness
	 (oneof (I1) (I2))
	 (not (I3)) (not (I4)) (not (I5)) 

	 ;; The patient isn't dead yet.
	 (not (D)))
  (:goal (and (not (I1)) (not (I2)) (not (I3)) (not (I4)) (not (I5))
	      (not (D)))))

(define (problem med3)
    (:domain med)
  
  (:init (not (R)) (not (B)) (not (W))	; Culture hasn't been done yet
	 (not (high-white-cell-count))	; Blood sample hasn't been taken.

	 ;; The patient has exactly one illness
	 (oneof (I1) (I2) (I3))
	 (not (I4)) (not (I5)) 

	 ;; The patient isn't dead yet.
	 (not (D)))
  (:goal (and (not (I1)) (not (I2)) (not (I3)) (not (I4)) (not (I5))
	      (not (D)))))

(define (problem med4)
    (:domain med)
  
  (:init (not (R)) (not (B)) (not (W))	; Culture hasn't been done yet
	 (not (high-white-cell-count))	; Blood sample hasn't been taken.

	 ;; The patient has exactly one illness
	 (oneof (I1) (I2) (I3) (I4))
	 (not (I5)) 

	 ;; The patient isn't dead yet.
	 (not (D)))
  (:goal (and (not (I1)) (not (I2)) (not (I3)) (not (I4)) (not (I5))
	      (not (D)))))

(define (problem med5)
    (:domain med)
  
  (:init (not (R)) (not (B)) (not (W))	; Culture hasn't been done yet
	 (not (high-white-cell-count))	; Blood sample hasn't been taken.

	 ;; The patient has exactly one illness
	 (oneof (I1) (I2) (I3) (I4) (I5))

	 ;; The patient isn't dead yet.
	 (not (D)))
  (:goal (and (not (I1)) (not (I2)) (not (I3)) (not (I4)) (not (I5))
	      (not (D)))))