(in-package :domains)

;;; This file contains some domains that exploit uncertainty:
;;;  - bt       Bomb in the toilet with single toilet, no clogging
;;;  - bmt      Bomb in the toilet with multiple toilets, no clogging
;;;  - btc      Bomb in the toilet with clogging
;;;  - bcts     like bct, except allows multiple toilets
;;;  - bt-asym  demonstrates asymmetric subgoals (different subgoal structure
;;;             in different contexts). Also demonstrates confrontation
;;;  - bt-sensory-?SA
;;;             adds sensory actions (? of them, where ? ranges from 1 to 4).

(define (domain bt)
  (:requirements :strips :equality :typing :conditional-effects :uncertainty)
  (:action dunk
	   :parameters (?p)
	   :precondition (and (package ?p) (not (dunked ?p))) ; Need quant to put this in when
	   :effect (and 
		    (when (in ?p) (defused))
		    (dunked ?p))))

;; Problem: BT(2)
(define (problem bt_2)              ; Plan length = 2
  (:domain bt)
  (:objects p1 p2)
  (:init (package p1) (package p2) (not (dunked p1)) (not (dunked p2)) 
         (oneof (in p1) (in p2)))
  (:goal (defused)))

;; Problem: BT(4)
(define (problem bt_4)              ; 
  (:domain bt)
  (:objects p1 p2 p3 p4)
  (:init (package p1) (package p2) (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
         (oneof (in p1) (in p2) (in p3) (in p4)))
  (:goal (defused)))

;; Problem: BT(6)
(define (problem bt_6)             
  (:domain bt)
  (:objects p1 p2 p3 p4 p5 p6)
  (:init (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
         (oneof (in p1) (in p2) (in p3) (in p4) (in p5) (in p6)))
  (:goal (defused)))

;; Problem: BT(8)
(define (problem bt_8)             
  (:domain bt)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8)
  (:init (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)))
  (:goal (defused)))

;; Problem: BT(10)
(define (problem bt_10)             
  (:domain bt)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10)
  (:init (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8) (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
	 (not (dunked p9)) (not (dunked p10)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10)))
  (:goal (defused)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BMT: Bomb in the toilet with multiple toilets, no clogging
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bmt)
  (:requirements :strips :equality :typing :conditional-effects :uncertainty)
  (:action dunk
             :parameters (?p ?t)
             :precondition (and (toilet ?t) (package ?p) 
                                (not (dunked ?p)))
             :effect (and (dunked ?p) 
			  (when (in ?p) (defused)))))


;; Problem: BMT(2,2)
(define (problem bmt_2_2)
  (:domain bmt)
  (:objects p1 p2 t1 t2)
  (:init (toilet t1) (toilet t2)
	 (package p1) (package p2) 
	 (not (dunked p1)) (not (dunked p2)) 
         (oneof (in p1) (in p2)))
  (:goal (defused)))

;; Problem: BMT(4,2)
(define (problem bmt_4_2)              ; 
  (:domain bmt)
  (:objects p1 p2 p3 p4 t1 t2)
  (:init (toilet t1) (toilet t2)
	 (package p1) (package p2) (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
         (oneof (in p1) (in p2) (in p3) (in p4)))
  (:goal (defused)))

;; Problem: BMT(6,2)
(define (problem bmt_6_2)             
  (:domain bmt)
  (:objects p1 p2 p3 p4 p5 p6 t1 t2)
  (:init (toilet t1) (toilet t2)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
         (oneof (in p1) (in p2) (in p3) (in p4) (in p5) (in p6)))
  (:goal (defused)))

;; Problem: BMT(8,4)
(define (problem bmt_8_4)             
  (:domain bmt)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 t1 t2 t3 t4)
  (:init (toilet t1) (toilet t2)
	 (toilet t3) (toilet t4)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)))
  (:goal (defused)))

;; Problem: BMT(10,4)
(define (problem bmt_10_4)             
  (:domain bmt)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10
	    t1 t2 t3 t4)
  (:init (toilet t1) (toilet t2)
	 (toilet t3) (toilet t4)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8) (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
	 (not (dunked p9)) (not (dunked p10)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BMT ;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BTC: Bomb in the toilet with single toilet, clogging
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain btc)
  (:requirements :strips :equality :typing :conditional-effects :uncertainty)
  (:action dunk
             :parameters (?p)
             :precondition (and (package ?p) (not (clogged)) 
				(not (dunked ?p))) 
             :effect (and (clogged) (dunked ?p)
		           (when (in ?p)
		             (defused))))
  (:action flush
             :parameters ()
	     :precondition (clogged)
             :effect (not (clogged))))

;; Problem: BTC(2)
(define (problem btc_2)              ; Plan length = 2
  (:domain btc)
  (:objects p1 p2)
  (:init (not (clogged))(package p1) (package p2) 
	 (not (dunked p1)) (not (dunked p2)) 
         (oneof (in p1) (in p2)))
  (:goal (defused)))

;; Problem: BTC(4)
(define (problem btc_4)              ; 
  (:domain btc)
  (:objects p1 p2 p3 p4)
  (:init (not (clogged))(package p1) (package p2) (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
         (oneof (in p1) (in p2) (in p3) (in p4)))
  (:goal (defused)))

;; Problem: BTC(6)
(define (problem btc_6)             
  (:domain btc)
  (:objects p1 p2 p3 p4 p5 p6)
  (:init (not (clogged))(package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
         (oneof (in p1) (in p2) (in p3) (in p4) (in p5) (in p6)))
  (:goal (defused)))

;; Problem: BTC(8)
(define (problem btc_8)             
  (:domain btc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8)
  (:init (not (clogged))(package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)))
  (:goal (defused)))

;; Problem: BTC(10)
(define (problem btc_10)             
  (:domain btc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10)
  (:init (not (clogged))(package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8) (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
	 (not (dunked p9)) (not (dunked p10)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BTC ;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BMTC: Bomb in the toilet with multiple toilets, clogging
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bmtc)
  (:requirements :strips :equality :typing :conditional-effects :uncertainty)
  (:action dunk
             :parameters (?p ?t)
             :precondition (and (package ?p) (toilet ?t)
                                (not (dunked ?p)) (not (clogged ?t)))
             :effect (and (clogged ?t) (dunked ?p)
		           (when (in ?p)
		             (defused))))
  (:action flush
             :parameters (?t)
             :precondition (and (toilet ?t))
             :effect (not (clogged ?t))))

;; Problem: BMTC(2,2)
(define (problem bmtc_2_2)
  (:domain bmtc)
  (:objects p1 p2 t1 t2)
  (:init (not (clogged t1)) (not (clogged t2))
	 (toilet t1) (toilet t2)
	 (package p1) (package p2)
	 (not (dunked p1)) (not (dunked p2)) 
         (oneof (in p1) (in p2)))
  (:goal (defused)))

;; Problem: BMTC(4,2)
(define (problem bmtc_4_2)              ; 
  (:domain bmtc)
  (:objects p1 p2 p3 p4 t1 t2)
  (:init (not (clogged t1)) (not (clogged t2))
	 (toilet t1) (toilet t2)
	 (package p1) (package p2) (package p3) (package p4)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
         (oneof (in p1) (in p2) (in p3) (in p4)))
  (:goal (defused)))

;; Problem: BMTC(6)
(define (problem bmtc_6_2)
  (:domain bmtc)
  (:objects p1 p2 p3 p4 p5 p6 t1 t2)
  (:init (not (clogged t1)) (not (clogged t2))
	 (toilet t1) (toilet t2)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
         (oneof (in p1) (in p2) (in p3) (in p4) (in p5) (in p6)))
  (:goal (defused)))

;; Problem: BMTC(8,4)
(define (problem bmtc_8_4)             
  (:domain bmtc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 t1 t2 t3 t4)
  (:init (not (clogged t1)) (not (clogged t2))
	 (not (clogged t3)) (not (clogged t4))
	 (toilet t1) (toilet t2)
	 (toilet t3) (toilet t4)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)))
  (:goal (defused)))

;; Problem: BMTC(10,4)
(define (problem bmtc_10_4)             
  (:domain bmtc)
  (:objects p1 p2 p3 p4 p5 p6 p7 p8 p9 p10
	    t1 t2 t3 t4)
  (:init (not (clogged t1)) (not (clogged t2))
	 (not (clogged t3)) (not (clogged t4))
	 (toilet t1) (toilet t2)
	 (toilet t3) (toilet t4)
	 (package p1) (package p2) (package p3) 
	 (package p4) (package p5) (package p6)
	 (package p7) (package p8) (package p9) (package p10)
	 (not (dunked p1)) (not (dunked p2)) 
	 (not (dunked p3)) (not (dunked p4)) 
	 (not (dunked p5)) (not (dunked p6)) 
	 (not (dunked p7)) (not (dunked p8)) 
	 (not (dunked p9)) (not (dunked p10)) 
         (oneof (in p1) (in p2) (in p3) (in p4) 
		(in p5) (in p6) (in p7) (in p8)
		(in p9) (in p10)))
  (:goal (defused)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BMTC ;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DOMAIN BMTC: Bomb in the toilet with clogging & one sensing action
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bts1) (:requirements :typing)
  (:action dunk
	   :parameters (?p)
	   :precondition (and (package ?p) (not (clogged)) 
						  (not (dunked ?p)))
	   :effect (and (clogged)
					(when (in ?p) (defused))))
  
  (:action detect-metal
	   :parameters (?p)
	   :precondition (package ?p)
	   :effect (observes (in ?p)))
)

(define (problem bts1-2) 
  (:domain bts1)
  (:objects pkg1 pkg2 pkg3 pkg4 pkg5 toilet1)
  (:init (not (clogged toilet1))
	 (toilet toilet1)
	 (armed)
	 (oneof (in pkg1) (in pkg2) (in pkg3) (in pkg4) (in pkg5))

	 (iff (in pkg1) (metal pkg1))
	 (iff (in pkg2) (metal pkg2))
	 (iff (in pkg3) (metal pkg3))
	 (iff (in pkg4) (metal pkg4))
	 (iff (in pkg5) (metal pkg5))
	 )

  (:goal (not (armed))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; End of BTS1 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (problem bct-df1)              
  (:domain bct)
  (:objects b p1)
  (:init (package p1)
         (bomb B) (not (clogged))
         (in p1 b))
  (:goal (defused b)))

(define (problem bct-df2)              
  (:domain bct)
  (:objects b p1 p2)
  (:init (package p1) (package p2)
         (bomb B) (not (clogged))
         (oneof (in p1 b) (in p2 b)))
  (:goal (defused b)))

(define (problem bct-df3)              
  (:domain bct)
  (:objects b p1 p2 p3)
  (:init (package p1) (package p2) (package p3)
         (bomb B) (not (clogged))
         (oneof (in p1 b) (in p2 b) (in p3 b)))
  (:goal (defused b)))

(define (problem bct-df4)              
  (:domain bct)
  (:objects b p1 p2 p3 p4)
  (:init (package p1) (package p2) (package p3) (package p4)
         (bomb B) (not (clogged))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b)))
  (:goal (defused b)))

(define (problem bct-df5)              ; five dunks; four flushes
  (:domain bct)
  (:objects b p1 p2 p3 p4 p5)
  (:init (package p1) (package p2) (package p3) (package p4) (package p5)
         (bomb B) (not (clogged))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b)))
  (:goal (defused b)))

(define (problem bct-df6)              ; five dunks; four flushes
  (:domain bct)
  (:objects b p1 p2 p3 p4 p5 p6)
  (:init (package p1) (package p2) (package p3) (package p4) (package p5) (package p6)
         (bomb B) (not (clogged))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b) (in p6 b)))
  (:goal (defused b)))

(define (problem bct-df7)              ; five dunks; four flushes
  (:domain bct)
  (:objects b p1 p2 p3 p4 p5 p6 p7)
  (:init (package p1) (package p2) (package p3) (package p4) (package p5) (package p6) (package p7)
         (bomb B) (not (clogged))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b) (in p6 b) (in p7 b)))
  (:goal (defused b)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Like previous version except allows for multiple toilets
;;;

(define (domain bcts)
  (:requirements :strips :equality :typing :conditional-effects :uncertainty)
  (:action dunk
             :parameters (?p ?t)
             :precondition (and (toilet ?t) (package ?p) 
                                (bomb ?b) (not (clogged ?t))) ; Need quant to put this in when
             :effect (and (clogged ?t)
		           (when (in ?p ?b)
		             (defused ?b))))
  (:action flush
             :parameters (?t)
             :precondition (and (toilet ?t) (clogged ?t))
             :effect (not (clogged ?t))))

;;;;;;;;;;;;;;;;;;;;
;;; Single toilet

(define (problem bcts-t1p1)              
  (:domain bcts)
  (:objects b t1 p1)
  (:init (bomb B)
         (package p1)
         (toilet t1) (not (clogged t1))
         (in p1 b))
  (:goal (defused b)))

(define (problem bcts-t1p2)              
  (:domain bcts)
  (:objects b t1 p1 p2)
  (:init (bomb B)
         (package p1) (package p2)
         (toilet t1) (not (clogged t1))
         (oneof (in p1 b) (in p2 b)))
  (:goal (defused b)))

(define (problem bcts-t1p3)              
  (:domain bcts)
  (:objects b t1 p1 p2 p3)
  (:init (bomb B)
         (package p1) (package p2) (package p3)
         (toilet t1) (not (clogged t1))
         (oneof (in p1 b) (in p2 b) (in p3 b)))
  (:goal (defused b)))

(define (problem bcts-t1p4)              
  (:domain bcts)
  (:objects b t1 p1 p2 p3 p4)
  (:init (bomb B)
         (package p1) (package p2) (package p3) (package p4)
         (toilet t1) (not (clogged t1))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b)))
  (:goal (defused b)))

(define (problem bcts-t1p5)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 p1 p2 p3 p4 p5)
  (:init (bomb B)
         (package p1) (package p2) (package p3) (package p4) (package p5)
         (toilet t1) (not (clogged t1))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b)))
  (:goal (defused b)))

(define (problem bcts-t1p6)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 p1 p2 p3 p4 p5 p6)
  (:init (bomb B)
         (package p1) (package p2) (package p3) (package p4) (package p5) (package p6)
         (toilet t1) (not (clogged t1))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b) (in p6 b)))
  (:goal (defused b)))

(define (problem bcts-t1p7)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 p1 p2 p3 p4 p5 p6 p7)
  (:init (bomb B)
         (package p1) (package p2) (package p3) (package p4) (package p5) (package p6) (package p7)
         (toilet t1) (not (clogged t1))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b) (in p6 b) (in p7 b)))
  (:goal (defused b)))

;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Two toilets

(define (problem bcts-t2p2)              
  (:domain bcts)
  (:objects b t1 t2 p1 p2)
  (:init (package p1) (package p2) (bomb B)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2))
         (oneof (in p1 b) (in p2 b)))
  (:goal (defused b)))

(define (problem bcts-t2p3)              
  (:domain bcts)
  (:objects b t1 t2 p1 p2 p3)
  (:init (package p1) (package p2) (package p3) (bomb B)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2))
         (oneof (in p1 b) (in p2 b) (in p3 b)))
  (:goal (defused b)))

(define (problem bcts-t2p4)              
  (:domain bcts)
  (:objects b t1 t2 p1 p2 p3 p4)
  (:init (package p1) (package p2) (package p3) (package p4) (bomb B)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b)))
  (:goal (defused b)))


(define (problem bcts-t2p5)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 t2 p1 p2 p3 p4 p5)
  (:init (package p1) (package p2) (package p3) (package p4) (package p5) (bomb B)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b)))
  (:goal (defused b)))

(define (problem bcts-t2p6)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 t2 p1 p2 p3 p4 p5 p6)
  (:init (bomb B)
         (package p1) (package p2) (package p3) (package p4) (package p5) (package p6)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b) (in p6 b)))
  (:goal (defused b)))

(define (problem bcts-t2p7)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 t2 p1 p2 p3 p4 p5 p6 p7)
  (:init (bomb B)
         (package p1) (package p2) (package p3) (package p4) (package p5) (package p6) (package p7)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b) (in p6 b) (in p7 b)))
  (:goal (defused b)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Three toilets

(define (problem bcts-t3p2)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 t2 t3 p1 p2)
  (:init (bomb B)
         (package p1) (package p2)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2)) (toilet t3) (not (clogged t3))
         (oneof (in p1 b) (in p2 b)))
  (:goal (defused b)))

(define (problem bcts-t3p3)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 t2 t3 p1 p2 p3)
  (:init (bomb B)
         (package p1) (package p2) (package p3)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2)) (toilet t3) (not (clogged t3))
         (oneof (in p1 b) (in p2 b) (in p3 b)))
  (:goal (defused b)))

(define (problem bcts-t3p4)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 t2 t3 p1 p2 p3 p4)
  (:init (bomb B)
         (package p1) (package p2) (package p3) (package p4)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2)) (toilet t3) (not (clogged t3))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b)))
  (:goal (defused b)))

(define (problem bcts-t3p5)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 t2 t3 p1 p2 p3 p4 p5)
  (:init (bomb B)
         (package p1) (package p2) (package p3) (package p4) (package p5)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2)) (toilet t3) (not (clogged t3))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b)))
  (:goal (defused b)))

(define (problem bcts-t3p6)              ; five dunks; four flushes
  (:domain bcts)
  (:objects b t1 t2 t3 p1 p2 p3 p4 p5 p6)
  (:init (bomb B)
         (package p1) (package p2) (package p3) (package p4) (package p5) (package p6)
         (toilet t1) (not (clogged t1)) (toilet t2) (not (clogged t2)) (toilet t3) (not (clogged t3))
         (oneof (in p1 b) (in p2 b) (in p3 b) (in p4 b) (in p5 b) (in p6 b)))
  (:goal (defused b)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (domain bt-asym)
  (:requirements :strips :equality  :typing :conditional-effects :uncertainty)
  (:action dunka
             :parameters ()
             :precondition (and (enabled) (not (clogged))) 
             :effect (when (bomb-in-a) 
                       (defused)))
  (:action dunkb
             :parameters ()
             :precondition (enabled)
             :effect (and (when (bomb-in-b)
		             (defused))
		           (when (and (not (bomb-in-b)) (not (pumping)))
		             (clogged))))
  (:action enable
             :parameters ()
             :precondition ()
             :effect (enabled))
  (:action pump
             :parameters ()
             :precondition ()
             :effect (pumping)))

(define (problem bta-c) 
  (:domain bt-asym)
  (:init (not (clogged)) (not (enabled)) (not (pumping))
         (oneof (bomb-in-a) (bomb-in-b)))
  (:goal (defused)))



(define (domain bt-sensory-1SA) (:requirements :typing)
  (:action dunk
	   :parameters (?pkg ?toilet)
	   :precondition (and (toilet ?toilet)
			      (object ?pkg))
	   :effect (and (clogged ?toilet)
			(when (and (in ?pkg)
				   (not (clogged ?toilet)))
			  (not (armed)))))
  
  (:action detect-metal
	   :parameters (?pkg)
	   :precondition (object ?pkg)
	   :effect (observes (metal ?pkg)))
)


(define (problem bt-1sa) (:domain bt-sensory-1SA)
  (:objects pkg1 pkg2 pkg3 pkg4 pkg5 toilet1)
  (:init (not (clogged toilet1))
	 (toilet toilet1)
	 (armed)
	 (oneof (in pkg1) (in pkg2) (in pkg3) (in pkg4) (in pkg5))

	 (iff (in pkg1) (metal pkg1))
	 (iff (in pkg2) (metal pkg2))
	 (iff (in pkg3) (metal pkg3))
	 (iff (in pkg4) (metal pkg4))
	 (iff (in pkg5) (metal pkg5))
	 )

  (:goal (not (armed))))


(define (domain bt-sensory-2SA) (:requirements :typing)
  (:action dunk
	   :parameters (?pkg ?toilet)
	   :precondition (and (toilet ?toilet)
			      (object ?pkg))
	   :effect (and (clogged ?toilet)
			(when (and (in ?pkg)
				   (not (clogged ?toilet)))
			  (not (armed)))))
  
  (:action detect-metal
	   :parameters (?pkg)
	   :precondition (object ?pkg)
	   :effect (observes (metal ?pkg)))

  ;; Use the bomb-sniffing dog.
  (:action sniff
	   :parameters (?pkg)
	   :precondition (object ?pkg)
	   :effect (observes (odor-of-explosives ?pkg)))
)


(define (problem bt-2sa) (:domain bt-sensory-2SA)
  (:objects pkg1 pkg2 pkg3 pkg4 pkg5 toilet1)
  (:init (not (clogged toilet1))
	 (toilet toilet1)
	 (armed)
	 (oneof (in pkg1) (in pkg2) (in pkg3) (in pkg4) (in pkg5))

	 (iff (in pkg1) (metal pkg1))
	 (iff (in pkg2) (metal pkg2))
	 (iff (in pkg3) (metal pkg3))
	 (iff (in pkg4) (metal pkg4))
	 (iff (in pkg5) (metal pkg5))

	 (iff (in pkg1) (odor-of-explosives pkg1))
	 (iff (in pkg2) (odor-of-explosives pkg2))
	 (iff (in pkg3) (odor-of-explosives pkg3))
	 (iff (in pkg4) (odor-of-explosives pkg4))
	 (iff (in pkg5) (odor-of-explosives pkg5))
	 )

  (:goal (not (armed))))



(define (domain bt-sensory-3SA) (:requirements :typing)
  (:action dunk
	   :parameters (?pkg ?toilet)
	   :precondition (and (toilet ?toilet)
			      (object ?pkg))
	   :effect (and (clogged ?toilet)
			(when (and (in ?pkg)
				   (not (clogged ?toilet)))
			  (not (armed)))))
  
  (:action detect-metal
	   :parameters (?pkg)
	   :precondition (object ?pkg)
	   :effect (observes (metal ?pkg)))

  ;; Use the bomb-sniffing dog.
  (:action sniff
	   :parameters (?pkg)
	   :precondition (object ?pkg)
	   :effect (observes (odor-of-explosives ?pkg)))

  (:action xray
	   :parameters (?pkg)
	   :precondition (object ?pkg)
	   :effect (observes (dense-material ?pkg)))
)


(define (problem bt-3sa) (:domain bt-sensory-3SA)
  (:objects pkg1 pkg2 pkg3 pkg4 pkg5 toilet1)
  (:init (not (clogged toilet1))
	 (toilet toilet1)
	 (armed)
	 (oneof (in pkg1) (in pkg2) (in pkg3) (in pkg4) (in pkg5))

	 (iff (in pkg1) (metal pkg1))
	 (iff (in pkg2) (metal pkg2))
	 (iff (in pkg3) (metal pkg3))
	 (iff (in pkg4) (metal pkg4))
	 (iff (in pkg5) (metal pkg5))

	 (iff (in pkg1) (odor-of-explosives pkg1))
	 (iff (in pkg2) (odor-of-explosives pkg2))
	 (iff (in pkg3) (odor-of-explosives pkg3))
	 (iff (in pkg4) (odor-of-explosives pkg4))
	 (iff (in pkg5) (odor-of-explosives pkg5))

	 (iff (in pkg1) (dense-material pkg1))
	 (iff (in pkg2) (dense-material pkg2))
	 (iff (in pkg3) (dense-material pkg3))
	 (iff (in pkg4) (dense-material pkg4))
	 (iff (in pkg5) (dense-material pkg5))
	 )

  (:goal (not (armed))))



(define (domain bt-sensory-4SA) (:requirements :typing)
  (:action dunk
	   :parameters (?pkg ?toilet)
	   :precondition (and (toilet ?toilet)
			      (object ?pkg))
	   :effect (and (clogged ?toilet)
			(when (and (in ?pkg)
				   (not (clogged ?toilet)))
			  (not (armed)))))
  
  (:action detect-metal
	   :parameters (?pkg)
	   :precondition (object ?pkg)
	   :effect (observes (metal ?pkg)))

  ;; Use the bomb-sniffing dog.
  (:action sniff
	   :parameters (?pkg)
	   :precondition (object ?pkg)
	   :effect (observes (odor-of-explosives ?pkg)))

  (:action xray
	   :parameters (?pkg)
	   :precondition (object ?pkg)
	   :effect (observes (dense-material ?pkg)))

  (:action listen-for-ticking
	   :parameters (?pkg)
	   :precondition (object ?pkg)
	   :effect (observes (ticking ?pkg)))
)


(define (problem bt-4sa) (:domain bt-sensory-4SA)
  (:objects pkg1 pkg2 pkg3 pkg4 pkg5 toilet1)
  (:init (not (clogged toilet1))
	 (toilet toilet1)
	 (armed)
	 (oneof (in pkg1) (in pkg2) (in pkg3) (in pkg4) (in pkg5))

	 (iff (in pkg1) (metal pkg1))
	 (iff (in pkg2) (metal pkg2))
	 (iff (in pkg3) (metal pkg3))
	 (iff (in pkg4) (metal pkg4))
	 (iff (in pkg5) (metal pkg5))

	 (iff (in pkg1) (odor-of-explosives pkg1))
	 (iff (in pkg2) (odor-of-explosives pkg2))
	 (iff (in pkg3) (odor-of-explosives pkg3))
	 (iff (in pkg4) (odor-of-explosives pkg4))
	 (iff (in pkg5) (odor-of-explosives pkg5))

	 (iff (in pkg1) (dense-material pkg1))
	 (iff (in pkg2) (dense-material pkg2))
	 (iff (in pkg3) (dense-material pkg3))
	 (iff (in pkg4) (dense-material pkg4))
	 (iff (in pkg5) (dense-material pkg5))

	 (iff (in pkg1) (ticking pkg1))
	 (iff (in pkg2) (ticking pkg2))
	 (iff (in pkg3) (ticking pkg3))
	 (iff (in pkg4) (ticking pkg4))
	 (iff (in pkg5) (ticking pkg5))
	 )

  (:goal (not (armed))))
